import { ServiceIdentifier } from '../../instantiation/common/instantiation.js';
import { CancellationToken } from '../../../base/common/cancellation.js';
import { Event } from '../../../base/common/event.js';
import { IQuickAccessController } from './quickAccess.js';
import { IQuickInputButton, IQuickPickItem, QuickPickInput, IPickOptions, Omit, IInputOptions, IQuickPick, IInputBox, IQuickWidget, IQuickNavigateConfiguration, IKeyMods, IQuickInput } from './quickInput.js';

declare const IQuickInputService: ServiceIdentifier<IQuickInputService>;
interface IQuickInputService {
    readonly _serviceBrand: undefined;
    /**
    * Provides access to the back button in quick input.
    */
    readonly backButton: IQuickInputButton;
    /**
    * Provides access to the quick access providers.
    */
    readonly quickAccess: IQuickAccessController;
    /**
    * Allows to register on the event that quick input is showing.
    */
    readonly onShow: Event<void>;
    /**
    * Allows to register on the event that quick input is hiding.
    */
    readonly onHide: Event<void>;
    /**
    * Opens the quick input box for selecting items and returns a promise
    * with the user selected item(s) if any.
    */
    pick<T extends IQuickPickItem>(picks: Promise<QuickPickInput<T>[]> | QuickPickInput<T>[], options?: IPickOptions<T> & {
        canPickMany: true;
    }, token?: CancellationToken): Promise<T[] | undefined>;
    pick<T extends IQuickPickItem>(picks: Promise<QuickPickInput<T>[]> | QuickPickInput<T>[], options?: IPickOptions<T> & {
        canPickMany: false;
    }, token?: CancellationToken): Promise<T | undefined>;
    pick<T extends IQuickPickItem>(picks: Promise<QuickPickInput<T>[]> | QuickPickInput<T>[], options?: Omit<IPickOptions<T>, 'canPickMany'>, token?: CancellationToken): Promise<T | undefined>;
    /**
    * Opens the quick input box for text input and returns a promise with the user typed value if any.
    */
    input(options?: IInputOptions, token?: CancellationToken): Promise<string | undefined>;
    /**
    * Provides raw access to the quick pick controller.
    */
    createQuickPick<T extends IQuickPickItem>(options: {
        useSeparators: true;
    }): IQuickPick<T, {
        useSeparators: true;
    }>;
    createQuickPick<T extends IQuickPickItem>(options?: {
        useSeparators: boolean;
    }): IQuickPick<T, {
        useSeparators: false;
    }>;
    /**
    * Provides raw access to the input box controller.
    */
    createInputBox(): IInputBox;
    /**
    * Provides raw access to the quick widget controller.
    */
    createQuickWidget(): IQuickWidget;
    /**
    * Moves focus into quick input.
    */
    focus(): void;
    /**
    * Toggle the checked state of the selected item.
    */
    toggle(): void;
    /**
    * Navigate inside the opened quick input list.
    */
    navigate(next: boolean, quickNavigate?: IQuickNavigateConfiguration): void;
    /**
    * Navigate back in a multi-step quick input.
    */
    back(): Promise<void>;
    /**
    * Accept the selected item.
    *
    * @param keyMods allows to override the state of key
    * modifiers that should be present when invoking.
    */
    accept(keyMods?: IKeyMods): Promise<void>;
    /**
    * Cancels quick input and closes it.
    */
    cancel(): Promise<void>;
    /**
    * The current quick pick that is visible. Undefined if none is open.
    */
    currentQuickInput: IQuickInput | undefined;
}

export { IQuickInputService };
